/*
Lee.dr 2021-02
https://youtu.be/yBa1LhfGmKo
https://youtu.be/mqU-qbdaiqU
https://youtu.be/BHJmxiUM4vk
*/


#include <FastLED.h>

#define LED_PIN     7
#define NUM_LEDS    30*7
#define BRIGHTNESS  64
#define LED_TYPE    WS2811
#define COLOR_ORDER GRB
CRGB leds[NUM_LEDS];

#define UPDATES_PER_SECOND 100

CRGBPalette16 currentPalette;
TBlendType    currentBlending;

extern CRGBPalette16 myRedWhiteBluePalette;
extern const TProgmemPalette16 myRedWhiteBluePalette_p PROGMEM;


int strobe = 4; // strobe pins on digital 4
int res = 5; // reset pins on digital 5
const int buttonPin = 2;

int left[7]; // store band values in these arrays
int right[7];
int lHold[7];
int rHold[7];
int fade;
int band;
uint8_t colorIndex;
int seq;
int mode;

//    uint8_t brightness = 255;
uint8_t brightness = 120;

CRGB red  = CHSV( HUE_RED, 255, brightness);
CRGB orange  = CHSV( HUE_ORANGE, 255, brightness);
CRGB yellow  = CHSV( HUE_YELLOW, 255, brightness);
CRGB green  = CHSV( HUE_GREEN, 255, brightness);
CRGB aqua = CHSV( HUE_AQUA, 255, brightness);
CRGB blue  = CHSV( HUE_BLUE, 255, brightness);
CRGB purple = CHSV( HUE_PURPLE, 255, brightness);
CRGB white = CHSV( 160, 0, brightness);
CRGB rainbow[] = {red, orange, yellow, green, aqua, blue, purple};
CRGB rainbow2[] = { green, green, green, yellow, yellow, red, red};

int divisor;
int max;
int avgMax;
int avgCount;
int buttonState = 0;
int incomingByte = 0; // for incoming serial data
boolean done;
unsigned long startMillis;


void setup()
{
    
	Serial.begin(115200);
  fade = 500;
  delay( 3000 ); // power-up safety delay
  FastLED.addLeds<LED_TYPE, LED_PIN, COLOR_ORDER>(leds, NUM_LEDS).setCorrection( TypicalLEDStrip );
  FastLED.setBrightness(  BRIGHTNESS );
  
  currentPalette = RainbowColors_p;
  currentBlending = LINEARBLEND;

  digitalWrite(res, LOW); // reset low
  digitalWrite(strobe, HIGH); //pin 5 is RESET on the shield
  // left and right channel from center
  // larger 
	pinMode(res, OUTPUT); // reset
	pinMode(strobe, OUTPUT); // strobe
  pinMode(buttonPin, INPUT);
//  pinMode(buttonPin, INPUT_PULLUP);
  // larger divisor makes smaller graph
  divisor = 55;
  avgCount = 0;
  avgMax = 800;
  seq = 0;
  mode = 9;
  done = false;
  startMillis = millis();
}

void loop()
{
  if (Serial.available() > 0) {
    // read the incoming byte:
    incomingByte = Serial.read();

    // say what you got:
    Serial.print("I received: ");
    Serial.println(incomingByte, DEC);
    Serial.println(char(incomingByte));
    if (incomingByte > 40) {
      mode = int(char(incomingByte))-48;
      Serial.print("mode = ");
      Serial.println(mode);
      done = false;
      seq = 0;
    }
  }
  
	readMSGEQ7();
  for (band = 0; band < 7; band++) {
    if (left[band] > max) max = left[band];
    if (right[band] > max) max = right[band];
  }
  if (max/divisor >= 15) {
    divisor++;
  } 

  if (mode < 7) {
    // blank everything first
    for( int i = 0; i < NUM_LEDS; i++) {
        leds[i] = ColorFromPalette( currentPalette, colorIndex, 0, currentBlending);
    }    
  }


  if (mode <= 2) {
    // this is for the red peak indicators for mode 1 and 2
    for (band = 0; band < 7; band++) {
      if (left[band] >= lHold[band]) {
        lHold[band] = left[band];
      } else {
        lHold[band] = lHold[band] - 4;
      }
      if (right[band] >= rHold[band]) {
        rHold[band] = right[band];
      } else {
        rHold[band] = rHold[band] - 4;
      }
    }
  }
  
  if (mode == 2 || mode == 3) {
    // left and right from center in color 
    for (band = 0; band < 7; band++) {
      for(int i = 14; i > 14 - int(left[band]/divisor); i--) {
        leds[(band*30) + i] = rainbow[band];
      }
      for(int i = 15; i < 15 + int(right[band]/divisor); i++) {
        leds[(band*30) + i] = rainbow[band];
      }
    } 
  }

//    for (band = 0; band < 7; band++) {
//      for(int i = ((band*30) + 14); i > ((band*30) + 14) - int(left[band]/divisor); i--) {
//        leds[i] = rainbow[band];
//      }
//      for(int i = ((band*30) + 15); i < ((band*30) + 15) + int(right[band]/divisor); i++) {
//        leds[i] = rainbow[band];
//      }
//    } 


  if (mode <= 2) {
    // this is for the red peak indicators for mode 1 and 2
    for (band = 0; band < 7; band++) {
      if (left[band] >= lHold[band]) {
        lHold[band] = left[band];
      } else {
        lHold[band] = lHold[band] - 1;
      }
      if (right[band] >= rHold[band]) {
        rHold[band] = right[band];
      } else {
        rHold[band] = rHold[band] - 1;
      }
      leds[((band*30) + 14) - int(lHold[band]/divisor)] = rainbow[0];
      leds[((band*30) + 15) + int(rHold[band]/divisor)] = rainbow[0];    
    }
  }


  
  if (mode == 4) {
    // vertical bands 2 pixels wide with color based on band
    for (int band = 0; band < 7; band++){
      for (int i = 0; i < 7; i++) {
        if (left[band] > int(1100/7)*i ) {
          leds[(i * 30) + band*2] = rainbow[band];
          leds[(i * 30) + 1 + band*2] = rainbow[band];
        }
        if (right[band] > int(1100/7)*i ) {
          leds[(i * 30) + band*2 + 15] = rainbow[band];
          leds[(i * 30) + 1 + band*2 + 15] = rainbow[band];
        }
      }
    }
  }


  if (mode == 5) {
    // vertical bands 2 pixels wide with color based on intensity (rainbow2 scheme)
    for (int band = 0; band < 7; band++){
      for (int i = 0; i < 7; i++) {
        if (left[band] > int(1100/7)*i ) {
          leds[(i * 30) + band*2] = rainbow2[i];
          leds[(i * 30) + 1 + band*2] = rainbow2[i];
        }
        if (right[band] > int(1100/7)*i ) {
          leds[(i * 30) + band*2 + 15] = rainbow2[i];
          leds[(i * 30) + 1 + band*2 + 15] = rainbow2[i];
        }
      }
    }
  }


  if (mode == 6) {
    if (seq < NUM_LEDS) {
      leds[seq++] =   rainbow[3];
      delay(100);
    } else {
      seq = 0;
    }
  }

  if (mode == 7) {
    // test pattern
    for( int i = 0; i < NUM_LEDS; i++) {
      leds[i] = ColorFromPalette( currentPalette, colorIndex, 0, currentBlending);
    }    

    // 0, 30, 60, 90, 120, 150, 180, 1, 31, 
    if (seq < NUM_LEDS) {
      leds[(seq%7 * 30) + int(seq/7)] =   rainbow[0];
      seq++;
      delay(100);
    } else {
      seq = 0;
    }
  }

  if (mode == 8 && done == false) {
    done = true;
    // US-ish flag
    for (int i = 0; i < 10; i++) {
      for (int j = 0; j < 7; j++) {
        if ((i+j)%2 == 0) {
          leds[i+(j*30)] = blue;
        } else {
          leds[i+(j*30)] = white;
        }
      }
    }
    for (int i = 10; i < 30; i++) {
      for (int j = 0; j < 7; j++) {
        if (j%2 == 0) {
          leds[i+(j*30)] = red; 
        } else {
          leds[i+(j*30)] = white;
        }
      }
    }
  }  


  if (mode == 10 && done == false) {
    done = true;
    // Texas flag
    for (int i = 0; i < 11; i++) {
      for (int j = 0; j < 7; j++) {
        leds[i+(j*30)] = blue;
      }
    }
    for (int i = 11; i < 30; i++) {
      for (int j = 0; j < 3; j++) {
        leds[i+(j*30)] = red; 
      }
      for (int j = 3; j < 7; j++) {
        leds[i+(j*30)] = white; 
      }      
    }
    int star[] = {183, 152, 153, 154, 119, 120, 121, 122, 123, 124, 125, 126, 127, 91,92, 93,94, 95, 61, 62, 64, 65, 30, 36};
    for (int i=0; i<sizeof star/sizeof star[0]; i++) {
      leds[star[i]+2] = white;
    }
  }  


  if (mode == 9) {
    // this is going to start with a single pixel in the far right column showing the amplitude of band with max value(in it's rainbox color)
    // then each pass shift everything left
    // so start with copying column 1 into column 0, up to column 28, then populate column 29 with fresh data
    for (int c = 1; c < 30; c++) {
      for (int r = 0; r < 7; r++) {
        leds[(c-1) + (r*30)] = leds[c + (r*30)];
      }
    }
    for( int i = 0; i < 7; i++) {
      leds[29  + (i*30)] = ColorFromPalette( currentPalette, colorIndex, 0, currentBlending);
    }    

    int maxBand = 0;
    int maxVal = 0;
    left[0] = left[0]*2;
    for (int band = 0; band < 7; band++) {
      if ((left[band]+right[band])/2 > maxVal) {
        maxBand = band;
        maxVal = (left[band]+right[band])/2;
      }
    }
    if (maxVal > 10) {
     leds[29  + ((int(8*maxVal/1024)-1)*30)] = rainbow[maxBand];
    }

    delay(5); 
  }  



  if (false) {
	// display values of left channel on serial monitor
  	for (band = 0; band < 7; band++)
  	{	
  		Serial.print(left[band]);
  		Serial.print(" ");
  	}
  	Serial.println(" - Left ");
  	// display values of right channel on serial monitor
  	for (band = 0; band < 7; band++)
  	{
  		Serial.print(right[band]);
  		Serial.print(" ");
  	}
  	Serial.println(" - Right" );
  }
  
  FastLED.show();

//  // read the state of the pushbutton value:
//  buttonState = digitalRead(buttonPin);
//  // check if the pushbutton is pressed. If it is, the buttonState is HIGH:
//  if (buttonState == HIGH) {
//    if (mode < 6) {
//      mode++;
//      Serial.print("mode = ");
//      Serial.println(mode);
//    } else {
//      mode = 1;
//    }
//  }
  
}

void readMSGEQ7()
// Function to read 7 band equalizers
{
	digitalWrite(res, HIGH);
	digitalWrite(res, LOW);
	for (band = 0; band < 7; band++)
	{
		digitalWrite(strobe, LOW); // strobe pin on the shield - kicks the IC up to the next band 
		delayMicroseconds(30); // 
		left[band] = analogRead(0); // store left band reading
		right[band] = analogRead(1); // ... and the right
		digitalWrite(strobe, HIGH);
    // clip very low/noise down to 0
    if (left[band] < 100) left[band] = 0;
    if (right[band] < 100) right[band] = 0;
	}
}
